'************************************************
'* MultiModem Class
'* Environment: Visual Basic 2005
'* Programmer: Sam Javner
'* Date: 4/29/2007
'*
'* Summary:
'*
'* Class for using features of the MultiTech MultiModem
'* Implements various commands from the reference guide of
'* AT Commands for MultiTech's Wireless GSM/GPRS Modems
'* Specifically, it implements commands for SMS Messaging
'* May be expandable to other modems which use the same AT Commands
'************************************************


'Won't have to fully qualify anything under the namespace System.IO.Ports
Imports System.IO.Ports


Public Class MultiModem

    'SerialPort object
    Private WithEvents SerialPort As New SerialPort


    'Keeps track of the current string between OnComm events
    Private current As String = String.Empty


    'Response Queue
    Private responseQ As New Queue(Of String)(50)

    'Incoming Message Queue
    Private incomingMsgQ As New Queue(Of String)(50)

    'List Messages Queue
    Private listMsgQ As New Queue(Of String)(50)
    Private listMsgFlag As Boolean = False

    'Lock variable, only send one AT Command at a time!
    Private lock As Integer = 0



    '************************************************
    '* Private Sub New()
    '* Summary:     Occurs when new instance of the class is created
    '*              Creates a new SerialPort object, initializes it
    '************************************************
    Public Sub New()

        '115200 baud rate, no parity, 8 data bits, 1 stop bit, no handshaking
        With SerialPort
            .BaudRate = 115200
            .Parity = IO.Ports.Parity.None
            .DataBits = 8
            .StopBits = 1
            .Handshake = Handshake.None
        End With

    End Sub



    '************************************************
    '* Public Function Connect(port As Integer) As Boolean
    '* Summary:     Attempts to connect to the serial port specified by port
    '* Parameters:  port - port number to connect to
    '* Returns:     True if successful, False if unsuccessful
    '************************************************
    Public Function Connect(ByVal port As Integer) As Boolean

        On Error Resume Next

        'Assign port number to PortName property
        SerialPort.PortName = "COM" & port

        'Try opening port
        SerialPort.Open()

        'Return True if port is open, False if port is not open
        Return SerialPort.IsOpen()

    End Function



    '************************************************
    '* Public Function Disconnect() As Boolean
    '* Summary:     Attempts to disconnect from the currently open serial port
    '* Returns:     True if successful or serial port already closed,
    '*              False if port is still open
    '************************************************
    Public Function Disconnect() As Boolean

        'Try closing the port
        SerialPort.Close()

        'Return True if port is closed, False if port is open
        Disconnect = Not SerialPort.IsOpen

    End Function


    '************************************************
    '* Private Sub SerialPort_DataReceived
    '* Summary:
    '* Event is raised whenever information is received from the serial port
    '* This routines parses the input into lines and places the lines into
    '* the appropriate queues depending on the content.
    '************************************************
    Private Sub SerialPort_DataReceived(ByVal sender As Object, ByVal e As System.IO.Ports.SerialDataReceivedEventArgs) Handles SerialPort.DataReceived

        Dim s As String
        Dim loc As Integer

        s = SerialPort.ReadExisting

        'Process all current input
        While Len(s) > 0

            'Current contains all unparsed input (variable is global to the class)
            current = current & s
            s = ""

            'Normally lines end with CR LF
            'However, when we are sending an outgoing SMS, we are prompted with "> "
            'For this reason, we will count "> " as its own line
            If InStr(current, "> ") = 1 Then

                'Add "> " to the response queue
                responseQ.Enqueue("> ")

                'Current contains all unparsed input
                current = Mid(current, 3)

            End If

            'If current contains CR LF then current contains a full line
            'Deal with the line if we have it
            If InStr(current, vbCrLf) > 0 Then

                loc = InStr(current, vbCrLf)

                's will contain all unparsed input
                s = Mid(current, loc + 2)

                'Current will contain the first full line from current
                current = Left(current, loc + 1)

                'Place s into the appropriate queue

                If InStr(current, "RING") = 1 Then

                    'Incoming phone call
                    'Ignore for now, could queue or do something else

                ElseIf InStr(current, "NO CARRIER") = 1 Then

                    'No Carrier message
                    'Ignore for now

                ElseIf InStr(current, "+CMTI:") = 1 Then

                    'Add to incoming message queue
                    incomingMsgQ.Enqueue(current)

                ElseIf InStr(current, "+CMGL:") = 1 Then

                    'Add to list messages queue, set flag to add next line as well
                    listMsgQ.Enqueue(current)
                    listMsgFlag = True

                ElseIf listMsgFlag Then

                    'Second line from list messages (+CMGL:)
                    listMsgQ.Enqueue(current)
                    listMsgFlag = False

                Else

                    'No reason to place it in any specific queue
                    'Place it in the response queue

                    If current <> vbCrLf Then
                        responseQ.Enqueue(current)
                    End If

                End If

                current = ""

            End If

        End While

    End Sub



    '************************************************
    '* Public Function SendATCommand(command As String, ByRef response As String, lines As Integer) As Boolean
    '* Summary:     Sends an AT Command to the serial port
    '* Parameters:  command - AT Command to send (no carriage return)
    '*              response - response from modem, if successful
    '*              lines - number of lines expected back from modem (not including any OKs)
    '* Returns:     True, if successful
    '*              False, if unsuccessful
    '************************************************
    Public Function SendATCommand(ByVal command As String, ByRef response As String, ByVal lines As Integer) As Boolean

        Dim ack As String
        Dim i As Integer

        'Assume unsuccesful
        SendATCommand = False

        'Exit if the port is not open
        If Not SerialPort.IsOpen Then Exit Function

        'Wait until other AT Commands are done executing
        While 1 = Threading.Interlocked.Exchange(lock, 1)
            Application.DoEvents()
        End While

        'Clear any "garbage" in the response queue
        responseQ.Clear()

        'Send the command followed by a carriage return
        SerialPort.Write(command & vbCr)

        'Wait for the specified number of lines, plus an OK or ERROR message
        While responseQ.Count < lines + 1
            Application.DoEvents()
        End While

        'Remove lines from the response queue and append to the variable response
        response = ""
        For i = 1 To lines
            response = response & responseQ.Dequeue
        Next i

        'Remove OK or ERROR message from the response queue
        ack = responseQ.Dequeue

        'ack should be "OK", if so then SendATCommand was successful
        If ack = "OK" & vbCrLf Then SendATCommand = True

        lock = 0

    End Function



    '************************************************
    '* Public Function SendMessage(phonenumber As String, msg As String) As Boolean
    '* Summary:     Sends a text mode SMS message
    '* Parameters:  phonenumber - phone number to send message to
    '*              msg - text of message
    '* Returns:     True, if successful
    '*              False, if unsuccessful
    '************************************************
    Public Function SendMessage(ByVal phonenumber As String, ByVal msg As String) As Boolean

        Dim prompt As String, response As String = String.Empty, ack As String

        'Assume unsuccessful
        SendMessage = False

        'Exit if the port is not open
        If Not SerialPort.IsOpen Then Exit Function

        'Set message format to text mode
        If Not SendATCommand("AT+CMGF=1", response, 0) Then Exit Function

        'Wait until other AT Commands are done executing
        While 1 = Threading.Interlocked.Exchange(lock, 1)
            Application.DoEvents()
        End While

        'Clear any "garbage" in the response queue
        responseQ.Clear()

        'Send message to phonenumber
        SerialPort.Write("AT+CMGS=" & phonenumber & vbCr)

        'Wait for a "> " prompt to output message
        While responseQ.Count < 1
            Application.DoEvents()
        End While

        prompt = responseQ.Dequeue

        If prompt = "> " Then

            'Output message followed by a Ctrl-Z (Chr(26))
            SerialPort.Write(msg & Chr(26))

            'Wait for a "+CMGS:..." response and an "OK" acknowledgement
            While responseQ.Count < 2
                Application.DoEvents()
            End While

            response = responseQ.Dequeue
            ack = responseQ.Dequeue

            If InStr(response, "+CMGS:") > 0 And ack = "OK" & vbCrLf Then SendMessage = True

        End If

        lock = 0

    End Function



    '************************************************
    '* Public Function SendMessageTPDU(phonenumber As String, msg As String) As Boolean
    '* Summary:     Sends a pdu mode SMS message
    '* Parameters:  phonenumber - phone number to send message to (make sure 10 digits!)
    '*              msg - message (as a hexadecimal text string)
    '* Returns:     True, if successful
    '*              False, if unsuccessful
    '************************************************
    Public Function SendMessageTPDU(ByVal phonenumber As String, ByVal msg As String) As Boolean

        Dim prompt As String, response As String = String.Empty, ack As String

        Dim fmsg As String
        Dim i As Integer

        'Assume unsuccessful
        SendMessageTPDU = False

        'Exit if the port is not open
        If Not SerialPort.IsOpen Then Exit Function

        'Set message format to pdu mode
        If Not SendATCommand("AT+CMGF=0", response, 0) Then Exit Function

        'Check documentation for a description of what is going in fmsg
        fmsg = "0001030A91"
        For i = 0 To 4
            fmsg = fmsg & Mid(phonenumber, i * 2 + 2, 1) & Mid(phonenumber, i * 2 + 1, 1)
        Next i

        fmsg = fmsg & "00F5"

        If (Len(msg) \ 2) < 16 Then
            fmsg = fmsg & "0" & Hex(Len(msg) \ 2)
        Else
            fmsg = fmsg & Hex(Len(msg) \ 2)
        End If

        fmsg = fmsg & msg



        'Wait until other AT Commands are done executing
        While 1 = Threading.Interlocked.Exchange(lock, 1)
            Application.DoEvents()
        End While

        'Clear any "garbage" in the response queue
        responseQ.Clear()

        'Send message of length = (length of fmsg - 2) divided by 2
        SerialPort.Write("AT+CMGS=" & ((Len(fmsg) - 2) \ 2) & vbCr)

        'Wait for a "> " prompt to output message
        While responseQ.Count < 1
            Application.DoEvents()
        End While

        prompt = responseQ.Dequeue

        If prompt = "> " Then

            'Output message followed by a Ctrl-Z (Chr(26))
            SerialPort.Write(fmsg & Chr(26))

            'Wait for a "+CMGS:..." response and an "OK" acknowledgement
            While responseQ.Count < 2
                Application.DoEvents()
            End While

            response = responseQ.Dequeue
            ack = responseQ.Dequeue

            If InStr(response, "+CMGS:") > 0 And ack = "OK" & vbCrLf Then SendMessageTPDU = True

        End If

        lock = 0

        'Set message format to text mode
        SendATCommand("AT+CMGF=1", response, 0)

    End Function



    '************************************************
    '* Public Function DeleteMessage(index As Integer) As Boolean
    '* Summary:     Deletes an SMS message from the modem
    '* Parameters:  index - index of message to delete
    '* Returns:     True if successful, False if unsuccessful
    '************************************************
    Public Function DeleteMessage(ByVal index As Integer) As Boolean

        Dim response As String = String.Empty

        DeleteMessage = False

        If Not SendATCommand("AT+CMGD=" & index, response, 0) Then Exit Function

        DeleteMessage = True

    End Function



    '************************************************
    '* Public Function GetClock(ByRef clock As String) As Boolean
    '* Summary:     Retrieves clock reading from modem
    '* Parameters:  clock - clock reading if successful
    '* Returns:     True if successful, False if unsuccessful
    '************************************************
    Public Function GetClock(ByRef clock As String) As Boolean

        Dim response As String = String.Empty

        GetClock = False

        If Not SendATCommand("AT+CCLK?", response, 1) Then Exit Function

        If InStr(response, vbCrLf) > 0 Then
            clock = Trim(Left(response, Len(response) - 2))
            If Len(clock) > 9 Then clock = Mid(clock, 9, Len(clock) - 9)
            GetClock = True
        End If

    End Function



    '************************************************
    '* Public Function GetManufacturerID(ByRef mid As String) As Boolean
    '* Summary:     Retrieves Manufacturer ID of modem
    '* Parameters:  manufacturer - Manufacturer ID if successful
    '* Returns:     True if successful, False if unsuccessful
    '************************************************
    Public Function GetManufacturerID(ByRef manufacturer As String) As Boolean

        Dim response As String = String.Empty

        GetManufacturerID = False

        If Not SendATCommand("AT+CGMI", response, 1) Then Exit Function

        If InStr(response, vbCrLf) > 0 Then
            manufacturer = Trim(Left(response, Len(response) - 2))
            GetManufacturerID = True
        End If

    End Function



    '************************************************
    '* Public Function GetModelID(ByRef model As String) As Boolean
    '* Summary:     Retrieves Model ID of modem
    '* Parameters:  model - Model ID if successful
    '* Returns:     True if successful, False if unsuccessful
    '************************************************
    Public Function GetModelID(ByRef model As String) As Boolean

        Dim response As String = String.Empty

        GetModelID = False

        If Not SendATCommand("AT+CGMM", response, 1) Then Exit Function

        If InStr(response, vbCrLf) > 0 Then
            model = Trim(Left(response, Len(response) - 2))
            GetModelID = True
        End If

    End Function



    '************************************************
    '* Public Function GetRevisionID(ByRef revision As String) As Boolean
    '* Summary:     Retrieves Revision ID of modem
    '* Parameters:  revision - Revision ID if successful
    '* Returns:     True if successful, False if unsuccessful
    '************************************************
    Public Function GetRevisionID(ByRef revision As String) As Boolean

        Dim response As String = String.Empty

        GetRevisionID = False

        If Not SendATCommand("AT+CGMR", response, 1) Then Exit Function

        If InStr(response, vbCrLf) > 0 Then
            revision = Trim(Left(response, Len(response) - 2))
            GetRevisionID = True
        End If

    End Function



    '************************************************
    '* Public Function GetSerialNumber(ByRef serial As String) As Boolean
    '* Summary:     Retrieves Serial Number of modem
    '* Parameters:  serial - Serial Number if successful
    '* Returns:     True if successful, False if unsuccessful
    '************************************************
    Public Function GetSerialNumber(ByRef serial As String) As Boolean

        Dim response As String = String.Empty

        GetSerialNumber = False

        If Not SendATCommand("AT+CGSN", response, 1) Then Exit Function

        If InStr(response, vbCrLf) > 0 Then
            serial = Trim(Left(response, Len(response) - 2))
            GetSerialNumber = True
        End If

    End Function



    '************************************************
    '* Public Function GetSignalQuality(ByRef strength As Integer) As Boolean
    '* Summary:     Determines the received signal strength
    '* Parameters:  strength - Signal strength if successful
    '*              Values:
    '*              0 -113 dBm or less
    '*              1 -111 dBm
    '*              2 to 30 -109 to 53 dBm
    '*              31 -51dBm or greater
    '*              99 not known or not detectable
    '* Returns:     True if successful, False if unsuccessful
    '************************************************
    Public Function GetSignalQuality(ByRef strength As Integer) As Boolean

        Dim response As String = String.Empty

        GetSignalQuality = False

        If Not SendATCommand("AT+CSQ", response, 1) Then Exit Function

        If InStr(response, "+CSQ:") = 1 Then
            response = Mid(response, 7)
            response = Left(response, InStr(response, ",") - 1)
            strength = CInt(response)
            GetSignalQuality = True
        End If

    End Function



    '************************************************
    '* Public Function GetSubscriberNumber(ByRef subscriber As String) As Boolean
    '* Summary:     Retrieves Subscriber Number of modem
    '* Parameters:  subscriber - Subscriber Number if successful
    '* Returns:     True if successful, False if unsuccessful
    '************************************************
    Public Function GetSubscriberNumber(ByRef subscriber As String) As Boolean

        Dim response As String = String.Empty

        GetSubscriberNumber = False

        If Not SendATCommand("AT+CNUM", response, 1) Then Exit Function

        If InStr(response, vbCrLf) > 0 Then
            subscriber = Trim(Left(response, Len(response) - 2))
            subscriber = Mid(subscriber, InStr(subscriber, """,") + 2)
            subscriber = Left(subscriber, InStr(subscriber, ",") - 1)
            subscriber = Mid(subscriber, 2, Len(subscriber) - 2)
            GetSubscriberNumber = True
        End If

    End Function



    '************************************************
    '* Public Function SetCharacterSet(characterset As String) As Boolean
    '* Summary:     Sets character set of modem for reading/writing messages in text mode
    '* Parameters:  characterset - Character Set to use
    '*              AT+CSCS=? will give a list of valid character sets, these include:
    '*              "GSM","PCCP437","CUSTOM","HEX"
    '*              "PCCP437" is the default
    '* Returns:     True if successful, False if unsuccessful
    '************************************************
    Public Function SetCharacterSet(ByVal characterset As String) As Boolean

        Dim response As String = String.Empty

        SetCharacterSet = False

        If Not SendATCommand("AT+CSCS=""" & characterset & """", response, 0) Then Exit Function

        SetCharacterSet = True

    End Function



    '************************************************
    '* Public Function SetClock(clock As String) As Boolean
    '* Summary:     Sets clock on modem
    '* Parameters:  clock - time to set modem to
    '* Returns:     True if successful, False if unsuccessful
    '************************************************
    Public Function SetClock(ByVal clock As String) As Boolean

        Dim response As String = String.Empty

        SetClock = False

        If Not SendATCommand("AT+CCLK=""" & clock & """", response, 0) Then Exit Function

        SetClock = True

    End Function



    '************************************************
    '* Public Function SetEcho(echo As Boolean) As Boolean
    '* Summary:     Sets echoing of AT commands to on or off
    '* Parameters:  echo - True, to turn echo on
    '*              False, to turn echo off
    '* Returns:     True if successful, False if unsuccessful
    '* Note:        Other functions in the MultiModem class
    '*              make the assumption that echoing is turned off
    '*              This function should be called after connecting
    '*              to the serial port to turn echoing off
    '************************************************
    Public Function SetEcho(ByVal echo As Boolean) As Boolean

        Dim response As String = String.Empty

        'Assume unsuccessful until we know that it is successful
        SetEcho = False

        'Exit function if the serial port is not open
        If Not SerialPort.IsOpen Then Exit Function

        'Turn echoing on or off
        If echo Then
            Call SendATCommand("ATE1", response, 0)
        Else
            Call SendATCommand("ATE0", response, 0)
        End If

        'Successful
        SetEcho = True

    End Function



    '************************************************
    '* Public Function SetNewMessageIndication(nmi As Integer) As Boolean
    '* Summary:     Sets behavior of New Message Indications
    '* Parameters:  nmi - behavior of New Message Indications
    '*              Values from documentation of AT Commands from Multitech:
    '*              0 No SMS-DELIVER indications are routed.
    '*              1 SMS-DELIVERs are routed using unsolicited code: +CMTI: SM,<index>
    '*              2 SMS-DELIVERs (except class 2 messages) are routed using unsolicited code: +CMT: ...
    '*              3 Class 3 SMS-DELIVERs are routed directly using code in <mt>=2
    '* Returns:     True if successful, False if unsuccessful
    '* Note:        nmi must be set to 1 for CheckMessages to work
    '*              One could write a "Check Messages" routine to work with nmi = 2
    '************************************************
    Public Function SetNewMessageIndication(ByVal nmi As Integer) As Boolean

        Dim response As String = String.Empty

        'Assume unsuccessful until we know that it is successful
        SetNewMessageIndication = False

        'Turn echoing on or off
        If Not SendATCommand("AT+CNMI=2," & nmi & ",0,0,0", response, 0) Then Exit Function

        'Successful
        SetNewMessageIndication = True

    End Function



    '************************************************
    '* Public Function ListMessages() As Boolean
    '* Summary:     Issues command to list all saved SMS messages
    '* Returns:     True
    '* Note:        Cannot call SendATCommand as is done in the other functions
    '*              SendATCommand reads a response of a determined number of lines
    '*              and then checks for an OK line. However, AT+CMGL does not
    '*              return an OK immediately. After issuing the command it will
    '*              return an indeterminate number of lines beginning with "+CMGL:"
    '*              depending on how many saved messages there are.
    '************************************************
    Public Function ListMessages() As Boolean

        'AT+CMGL="ALL" is used here.
        'AT+CMGL="READ" or AT+CMGL="UNREAD" are also valid uses of AT+CMGL

        'Wait until other AT Commands are done executing
        While 1 = Threading.Interlocked.Exchange(lock, 1)
            Application.DoEvents()
        End While

        SerialPort.Write("AT+CMGL=""ALL""" & vbCr)

        lock = 0

        ListMessages = True

    End Function



    '************************************************
    '* Public Function CheckListMessages(ByRef index As Integer, ByRef unread As Boolean,
    '*                 ByRef number As String, ByRef timestamp As String, ByRef msg As String)
    '* Summary:     Checks the list message queue
    '* Parameters:  index - index of message, if there is one
    '*              unread - True if message is unread, False if message is read
    '*              number - phone number (address) of sender
    '*              timestamp - time stamp of message
    '*              msg - contents of message
    '* Returns:     True if there is a message, False otherwise
    '************************************************
    Public Function CheckListMessages(ByRef index As Integer, ByRef unread As Boolean, ByRef number As String, ByRef timestamp As String, ByRef msg As String)

        Dim line1 As String, line2 As String, s As String
        Dim pos As Integer

        CheckListMessages = False

        'Needs to be 2 lines in the list message queue
        'First line contains index, read status, phone number, and timestamp
        'Second line contains contents of message
        If listMsgQ.Count >= 2 Then

            line1 = listMsgQ.Dequeue
            line2 = listMsgQ.Dequeue

            If InStr(line1, "+CMGL:") <> 1 Then Exit Function

            'Read index of message
            line1 = Mid(line1, 8)
            pos = InStr(line1, ",")
            s = Left(line1, pos - 1)
            index = CInt(s)

            'Read whether message has been read or not
            line1 = Mid(line1, pos + 1)
            pos = InStr(line1, ",")
            s = Left(line1, pos - 1)
            If s = """REC READ""" Then
                unread = False
            ElseIf s = """REC UNREAD""" Then
                unread = True
            Else
                'Shouldn't be here
                unread = True
            End If

            'Read phone number
            line1 = Mid(line1, pos + 1)
            pos = InStr(line1, ",")
            s = Left(line1, pos - 1)
            number = Mid(s, 2, Len(s) - 2)      'Strip quotes

            'This parameter appears to be blank and not used
            line1 = Mid(line1, pos + 1)
            pos = InStr(line1, ",")
            s = Left(line1, pos - 1)

            'Read time stamp (last parameter)
            line1 = Mid(line1, pos + 1)
            pos = InStr(line1, vbCrLf)
            s = Left(line1, pos - 1)
            timestamp = Mid(s, 2, Len(s) - 2)   'Strip quotes

            'Read message contents
            msg = line2
            If InStr(msg, vbCrLf) = 0 Then Exit Function
            msg = Left(msg, Len(msg) - 2)

            CheckListMessages = True

        End If

    End Function



    '************************************************
    '* Public Function CheckMessages(ByRef index As Integer) As Boolean
    '* Summary:     Checks the incoming message queue for new messages
    '* Parameters:  index - index of new message, if new messages
    '* Returns:     True if new messages, False if no new messages
    '************************************************
    Public Function CheckMessages(ByRef index As Integer) As Boolean

        Dim cmti As String
        Dim pos As Integer

        'Assume unsuccesful
        CheckMessages = False

        'Check incoming message queue
        If incomingMsgQ.Count >= 1 Then

            'Parse first message in queue for index
            cmti = incomingMsgQ.Dequeue
            pos = InStr(cmti, """,")
            cmti = Mid(cmti, pos + 2)
            cmti = Left(cmti, Len(cmti) - 2)
            index = CInt(cmti)

            CheckMessages = True

        End If

    End Function


    '************************************************
    '* Public Function ReadMessage(index As Integer, ByRef unread As Boolean,
    '*                 ByRef number As String, ByRef timestamp As String, ByRef msg As String)
    '* Summary:     Reads the SMS message at index
    '* Parameters:  index - index of message to read
    '*              unread - True if message is unread, False if message is read
    '*              number - phone number (address) of sender
    '*              timestamp - time stamp of message
    '*              msg - contents of message
    '* Returns:     True if successful, False if unsuccessful
    '************************************************
    Public Function ReadMessage(ByVal index As Integer, ByRef unread As Boolean, ByRef number As String, ByRef timestamp As String, ByRef msg As String)

        Dim line1 As String = String.Empty
        Dim pos As Integer
        Dim s As String

        ReadMessage = False

        If Not SendATCommand("AT+CMGR=" & index, line1, 2) Then Exit Function

        'Read whether message has been read or not
        line1 = Mid(line1, 8)
        pos = InStr(line1, ",")
        s = Left(line1, pos - 1)
        If s = """REC READ""" Then
            unread = False
        ElseIf s = """REC UNREAD""" Then
            unread = True
        Else
            'Shouldn't be here
            unread = True
        End If

        'Read phone number
        line1 = Mid(line1, pos + 1)
        pos = InStr(line1, ",")
        s = Left(line1, pos - 1)
        number = Mid(s, 2, Len(s) - 2)      'Strip quotes

        'This parameter appears to be blank and not used
        line1 = Mid(line1, pos + 1)
        pos = InStr(line1, ",")
        s = Left(line1, pos - 1)

        'Read time stamp (last parameter)
        line1 = Mid(line1, pos + 1)
        pos = InStr(line1, vbCrLf)
        s = Left(line1, pos - 1)
        timestamp = Mid(s, 2, Len(s) - 2)   'Strip quotes

        'Read message contents
        msg = Mid(line1, pos + 2)
        If InStr(msg, vbCrLf) = 0 Then Exit Function
        msg = Left(msg, Len(msg) - 2)

        ReadMessage = True

    End Function



End Class
